/*    callrec is a program that records calls on an openmoko.
 *    Copyright (C) 2009  Tom Hacohen (tom@stosb.com)
 *
 *    This file is a part of callrec
 * 
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stdlib.h> 
#include <string.h>
#include <sys/types.h>
#include <signal.h>
#include <time.h>
#include <unistd.h>

#include <stdio.h>
#include <errno.h>

#include "recording.h"


/* the sound filename generation specs */
#define FILENAME_PREFIX "call_"
#define FILENAME_PREFIX_LEN 5
#define FILENAME_POSTFIX ".wav"
#define FILENAME_POSTFIX_LEN 4
#define FILENAME_DATE_FORMAT "%m%d%y_%H%M%S"
#define FILENAME_TEMPLATE "DDMMYY_HHMMSS"
#define FILENAME_TEMPLATE_LEN 13

/* the pids of the playing/recording childs */
static pid_t playing_pid = 0;
static pid_t recording_pid = 0;

/* the templatep for the filename (date+postfix) */
static char file_template[] = FILENAME_TEMPLATE FILENAME_POSTFIX;/* the template */
#define FILE_TEMPLATE_LEN FILENAME_TEMPLATE_LEN + FILENAME_POSTFIX_LEN

/* the last file recorded */
static char *last_file = NULL;

/**
 * generate_last_file() - sets 'static char *last_file' to the a new unique filename.
 *
 * This function sets 'static char *last_file' to the a new unique filename.
 * The new filename looks like ~/FILENAME_PREFIX FILENAME_TEMPLATE FILENAME_POSTFIX
 * The function allocates the memory needed for last_file. Assumes that if last_file != NULL
 * it's a legit and big enough char string.
 **/
static int
generate_last_file(void)
{
	time_t nowbin;
	const struct tm *nowstruct;
	const char *home_dir = getenv("HOME");
	
	if (time(&nowbin) == (time_t) - 1)
		return -1;
	
	nowstruct = localtime(&nowbin);

	if (strftime(file_template, FILENAME_TEMPLATE_LEN+1, FILENAME_DATE_FORMAT, nowstruct) == (size_t) 0)
		return -2;

	/*FIXME: verify len add the postfix*/
	strcat(file_template, FILENAME_POSTFIX);
	
	if (last_file == NULL) {
		last_file = malloc(strlen(home_dir) + FILENAME_PREFIX_LEN + FILE_TEMPLATE_LEN + 1 + 1); /* one for '/' other for null */
		if (last_file == NULL) {
			return -3;
		}
		strcpy(last_file, home_dir);
		strcat(last_file, "/");
		strcat(last_file, FILENAME_PREFIX);
		strcat(last_file, file_template);
	}
	else {
		/*copy the template to the last file, modify only the last chars*/
		strcpy(&last_file[strlen(last_file) - FILE_TEMPLATE_LEN], file_template);
	}
	return 0;
}

/**
 * start_recording() - Starts recording.
 *
 * This function starts the recording to the file last_file (calls generate_last_file())
 * set recording_pid to the pid of the child process who's recording.
 *
 * upon success returns the pid 0 on error.
 **/
int
start_recording(void)
{
	if (generate_last_file()) 
		return 0;
		
	recording_pid = fork();
	
	if (recording_pid == 0) {
		int exec_ret = execl("/usr/bin/arecord", "arecord", "-D", "hw:0,0", "-f", "S16_LE", "-c", "2", "-t", "wav", last_file, NULL);
		printf("Recording to file: %s\n", last_file);
		if (exec_ret) {
			perror("arecord: ");
			exit(1);
		}
		else {
			exit(0);
		}
	}
	else if (recording_pid < 0) {
		recording_pid = 0;
		return 0;
	}
	return recording_pid;
}

/**
 * stop_recording() - Stops recording.
 *
 * This function stops the active recording session (recording_pid).
 * ignores if recording_pid is NULL or doesn't exist.
 *
 * also sets recording_id to 0 uppon completion.
 **/
int
stop_recording(void)
{
	if (recording_pid > 0) { 
		/* we don't care about the outcome of kill, all is good*/
		kill(recording_pid, SIGTERM);
	}
	
	recording_pid = 0;
	return 0;
}

/**
 * start_playing() - Starts playing.
 *
 * This function starts the playing the file last_file.
 * sets playing_pid to the pid of the child process who's recording.
 *
 * upon success returns the pid 0 on error.
 **/
int
start_playing(void)
{
	playing_pid = fork();
	
	if (playing_pid == 0) {
		int exec_ret = execl("/usr/bin/aplay", "aplay", "-D", "hw:0,0", last_file, NULL);
		printf("Playing file: %s\n", last_file);
		if (exec_ret) {
			perror("aplay: ");
			exit(1);
		}
		else {
			exit(0);
		}
	}
	else if (playing_pid < 0) {
		playing_pid = 0;
		return 0;
	}
	
	return playing_pid;
}

/**
 * stop_playing() - Stops playing.
 *
 * This function stops the active recording session (playing_pid).
 * ignores if playing_pid is NULL or doesn't exist.
 *
 * also sets playing_id to 0 uppon completion.
 **/
int
stop_playing(void)
{
	if (playing_pid > 0) { 
		/* we don't care about the outcome of kill, all is good*/
		kill(playing_pid, SIGTERM);	
	}
	
	playing_pid = 0;
	return 0;
}

/**
 * toggle_recording() - Toggles the recording status.
 *
 * This function toggles the recording status.
 *
 * returns 1 if it started recording, 0 otherwise.
 **/
int
toggle_recording(void)
{
	if (is_recording()) {
		stop_recording();
		return 0;
	}
	else {
		start_recording();
		return 1;
	}
}

/**
 * toggle_playing() - Toggles the playing status.
 *
 * This function toggles the playing status.
 *
 * returns 1 if it started playing, 0 otherwise.
 **/
int
toggle_playing(void)
{
	if (is_playing()) {
		stop_playing();
		return 0;
	}
	else {
		start_playing();
		return 1;
	}
}

/**
 * is_recording() - Returns the recording status.
 *
 * This function returns 1 if recording 0 otherwise.
 **/
int
is_recording(void)
{
	return (recording_pid) ? 1:0;
}

/**
 * is_playing() - Returns the playing status.
 *
 * This function returns 1 if playing 0 otherwise.
 **/
int
is_playing(void)
{
	return (playing_pid) ? 1:0;
}

/**
 * whois_pid() - Returns the type of pid.
 * @pid:	the pid to check.
 *
 * This function returns the type of the id passed as parameter.
 * type means the "job" (recording/playing) of the process who's pid is pid.
 **/
enum ActionType
whois_pid(pid_t pid)
{
	if (pid == playing_pid)
		return ACTION_PLAY;
	else if (pid == recording_pid)
		return ACTION_RECORD;
	else
		return ACTION_UNKNOWN;
}